unit Dxfvmain;

interface

uses
	SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
	Forms, Dialogs, Menus, ExtCtrls, Dxfgrph;

const
	DxfDir = 'd:\images\dxfs\utah\';

type
	TForm1 = class(TForm)
		MainMenu1: TMainMenu;
		OpenDialog1: TOpenDialog;
		File1: TMenuItem;
    Open1: TMenuItem;
		Test1: TMenuItem;
		PaintBox1: TPaintBox;
    CompactDirectory1: TMenuItem;
    N1: TMenuItem;
    N2: TMenuItem;
    Properties1: TMenuItem;
		procedure Open1Click(Sender: TObject);
		procedure Test1Click(Sender: TObject);
		procedure FormCreate(Sender: TObject);
		procedure FormDestroy(Sender: TObject);
		procedure PaintBox1Paint(Sender: TObject);
		procedure FormResize(Sender: TObject);
    procedure Properties1Click(Sender: TObject);
	private
		{ Private declarations }
	public
		{ Public declarations }
		Dxf: TDxf;
		procedure SizeAndPaintDxf;
	end;

var
	Form1: TForm1;

implementation

{$R *.DFM}

procedure TForm1.Open1Click(Sender: TObject);
begin
	if OpenDialog1.Execute then
		begin
			Dxf.ScaleFactor := 3;
			Dxf.LoadFromFile(OpenDialog1.FileName);
			PaintBox1.Invalidate;
		end;
end;

procedure TForm1.SizeAndPaintDxf;
var
	Scale1: Extended;
	ORect: TRect;
begin
	ORect := PaintBox1.Canvas.ClipRect;
	PaintBox1.Canvas.Brush.Color := clBtnFace;
	PaintBox1.Canvas.Brush.Style := bsSolid;
	PaintBox1.Canvas.FillRect(ORect);
	if Dxf.Empty then
		Exit;
	Dxf.Width := PaintBox1.Width - 10;
	Scale1 := Dxf.ScaleFactor;
	Dxf.Height := PaintBox1.Height - 10;
	if Scale1 < Dxf.ScaleFactor then
  	Dxf.ScaleFactor := Scale1;
  Dxf.MarkOrigin := True;
	PaintBox1.Canvas.Draw(5, 5, Dxf);
end;

procedure TForm1.Test1Click(Sender: TObject);
var
	Dxf: TDxf;
	ORect: TRect;
	SearchRec: TSearchRec;
  f: TFileStream;
	r: Integer;
	LoadTime, DrawTime, xTime, Count: DWord;
  Dir: String;
begin
  Screen.Cursor := crHourglass;
  try
    Dxf := TDxf.Create;
    Dxf.ScaleFactor := 3;
    LoadTime := 0;
    DrawTime := 0;
    Count := 0;
    Dir := ExtractFilePath(OpenDialog1.FileName);
    if Dir = '' then
      Dir := DxfDir;
    try
      r := FindFirst(Dir + '*.dxf', faAnyFile, SearchRec);
      while r = 0 do
        begin
          xTime := GetTickCount;
          try
            Dxf.LoadFromFile(Dir + SearchRec.Name);
          except
            on Exception do
              begin
                r := FindNext(SearchRec);
                continue;
              end;
          end;
          LoadTime := LoadTime + GetTickCount - xTime;

          if Sender = CompactDirectory1 then begin
            f := TFileStream.Create(Dir + Copy(SearchRec.Name, 1, Length(SearchRec.Name) - 4), fmCreate);
            try
              Dxf.SaveToStreamCompact(f);
            finally
              f.Free;
            end;
          end else begin
            ORect := ClientRect;
            Canvas.Brush.Color := clBtnFace;
            Canvas.Brush.Style := bsSolid;
            Canvas.FillRect(ORect);
            xTime := GetTickCount;
            Dxf.Height := ClientHeight;
            Canvas.Draw(4, 4, Dxf);
            DrawTime := DrawTime + GetTickCount - xTime;
          end;
          Inc(Count);
          r := FindNext(SearchRec);
        end;
      if (Count > 0) and (Sender = Test1) then
        MessageDlg('Load Time: ' + IntToStr(LoadTime div Count) + ' ms'#13
            + 'Draw Time: ' + IntToStr(DrawTime div Count) + ' ms', mtInformation, [mbOk], 0);
    except
      on Exception do
        begin
          Dxf.Free;
          raise;
        end;
    end;
    Dxf.Free;
  finally
    Screen.Cursor := crDefault;
  end;
end;

procedure TForm1.FormCreate(Sender: TObject);
begin
	Dxf := TDxf.Create;
	Resize;
end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
	Dxf.Free;
  Dxf := nil;
end;

procedure TForm1.PaintBox1Paint(Sender: TObject);
begin
	SizeAndPaintDxf;
end;

procedure TForm1.FormResize(Sender: TObject);
begin
	if Assigned (Dxf) and not Dxf.Empty then
		SizeAndPaintDxf;
end;

procedure TForm1.Properties1Click(Sender: TObject);
begin
	if Assigned (Dxf) and not Dxf.Empty then
    Application.MessageBox(PChar(Format('Extent: (%g, %g)-(%g, %g)',
        [Dxf.MinMappingExtent.X, Dxf.MinMappingExtent.Y,
        Dxf.MaxMappingExtent.X, Dxf.MaxMappingExtent.Y])), 'Properties')
end;

end.
